/*  
    pmacct (Promiscuous mode IP Accounting package)
    pmacct is Copyright (C) 2003 by Paolo Lucente
*/

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/* includes */
#include "pmacct.h"
#include "imt_plugin.h"

/* functions */
unsigned char *init_accounting_structure()
{
  struct acc *elem_acc;
  unsigned char *base_ptr;
  int i;

  /* base_ptr or ptr ? */
  base_ptr = current_pool->base_ptr;

  for (i = 0; i < config.buckets; i++) {
    if (current_pool->space_left >= sizeof(struct acc)) {
      elem_acc = (struct acc *) current_pool->ptr;
      elem_acc->ipa.s_addr = 0;
      elem_acc->packet_counter = 0;
      elem_acc->bytes_counter = 0;
      elem_acc->next = NULL;

      current_pool->ptr += sizeof(struct acc);
      current_pool->space_left -= sizeof(struct acc);
    }
    else printf("WARN: We finished space; initialized %d of %d config.buckets\n", i, config.buckets); 
  }
  
  return base_ptr;
}


struct acc *search_accounting_structure(struct pkt_primitives *addr)
{
  struct acc *elem_acc;
  unsigned char *elem;
  unsigned long int pos, pos2;
  int solved;

  /* Rich Gade modulo */
  pos = (addr->eth_dhost[ETH_ADDR_LEN-4] << 24) | (addr->eth_dhost[ETH_ADDR_LEN-3] << 16) | 
	(addr->eth_dhost[ETH_ADDR_LEN-2] << 8) | addr->eth_dhost[ETH_ADDR_LEN-1];
  pos ^= (addr->eth_shost[ETH_ADDR_LEN-1] << 24) | (addr->eth_shost[ETH_ADDR_LEN-2] << 16) |
	(addr->eth_shost[ETH_ADDR_LEN-3] << 8) | addr->eth_shost[ETH_ADDR_LEN-4];

  pos2 = addr->ipa.s_addr;
  pos2 ^= (addr->flow.s_addr << 8) | (addr->flow.s_addr >> 24);
  pos2 ^= ((unsigned int)addr->src_port << 16) | addr->dst_port | ((unsigned short int) addr->proto << 10);

  pos ^= pos2;
  pos %= config.buckets;

  elem = a;
  elem += pos * sizeof(struct acc);
  elem_acc = (struct acc *) elem;
  solved = FALSE;

  while (solved == FALSE) {
    if (memcmp(elem_acc, addr, sizeof(struct pkt_primitives)) == 0) return elem_acc;
    else if ((memcmp(elem_acc, addr, sizeof(struct pkt_primitives)) != 0) && (elem_acc->next != NULL)) {
      if (debug) printf("Walking through the collision-chain in this bucket.\n");
      elem_acc = elem_acc->next;
    }
    else if ((memcmp(elem_acc, addr, sizeof(struct pkt_primitives)) != 0) && (elem_acc->next == NULL))
      return NULL;
  } 

  return NULL;
}


struct acc *insert_accounting_structure(struct pkt_primitives *addr, int pk_len)
{
  struct acc *elem_acc;
  struct pkt_primitives empty_addr;
  unsigned char *elem, *new_elem;
  int solved;
  unsigned long int pos, pos2;

  elem = a;
  solved = FALSE;
  memset(&empty_addr, 0, sizeof(struct pkt_primitives));

  /* trivial method to get a modulo, trying to avoid long chains */
  // pos = (addr->ipa.s_addr+addr->flow.s_addr+addr->src_port+addr->dst_port) % config.buckets;

  /* Rich Gade modulo */
  pos = (addr->eth_dhost[ETH_ADDR_LEN-4] << 24) | (addr->eth_dhost[ETH_ADDR_LEN-3] << 16) |
	(addr->eth_dhost[ETH_ADDR_LEN-2] << 8) | addr->eth_dhost[ETH_ADDR_LEN-1];
  pos ^= (addr->eth_shost[ETH_ADDR_LEN-1] << 24) | (addr->eth_shost[ETH_ADDR_LEN-2] << 16) |
	(addr->eth_shost[ETH_ADDR_LEN-3] << 8) | addr->eth_shost[ETH_ADDR_LEN-4];

  pos2 = addr->ipa.s_addr;
  pos2 ^= (addr->flow.s_addr << 8) | (addr->flow.s_addr >> 24);
  pos2 ^= ((unsigned int)addr->src_port << 16) | addr->dst_port | ((unsigned short int) addr->proto << 10);

  pos ^= pos2;
  pos %= config.buckets;
      
  if (debug) printf("Bucket: %u\n", pos);
  elem += pos * sizeof(struct acc);
  elem_acc = (struct acc *) elem;

  while (solved == FALSE) {
    if (memcmp(elem_acc, addr, sizeof(struct pkt_primitives)) == 0) {
      elem_acc->packet_counter++;
      elem_acc->bytes_counter += pk_len;
      return (struct acc *) elem_acc;
    }
    else if (memcmp(elem_acc, &empty_addr, sizeof(struct pkt_primitives)) == 0) {
      memcpy(elem_acc, addr, sizeof(struct pkt_primitives));
      elem_acc->packet_counter++;
      elem_acc->bytes_counter += pk_len;
      return (struct acc *) elem_acc;
    }

    /* Handling collisions */
    // else if ((memcmp(&elem_acc->ipa, addr, sizeof(struct pkt_primitives)) != 0) && (elem_acc->next != NULL)) {
    else if (elem_acc->next != NULL) {
      if (debug) printf("Walking through the collision-chain in this bucket\n");
      elem_acc = elem_acc->next;
      solved = FALSE;
    }
    // else if ((memcmp(&elem_acc->ipa, addr, sizeof(struct pkt_primitives)) != 0) && (elem_acc->next == NULL)) {
    else if (elem_acc->next == NULL) {
      /* We have to know if there is enough space for a new element;
         if not we are losing informations; conservative approach */
      if (no_more_space) return NULL;

      /* We have to allocate new space for this address */
      if (debug) printf("Creating new element in this bucket\n");

      if (current_pool->space_left >= sizeof(struct acc)) {
        new_elem = current_pool->ptr;
	current_pool->space_left -= sizeof(struct acc);
	current_pool->ptr += sizeof(struct acc);
      }
      else {
        current_pool = request_memory_pool(config.memory_pool_size); 
	if (current_pool == NULL) {
          printf("WARN: Cannot allocate more memory pools, clear stats manually!\n");
	  no_more_space = TRUE;
	  return NULL;
        }
        else {
          new_elem = current_pool->ptr;
          current_pool->space_left -= sizeof(struct acc);
          current_pool->ptr += sizeof(struct acc);
	}
      }

      elem_acc->next = (struct acc *) new_elem;
      elem_acc = (struct acc *) new_elem;
      memcpy(elem_acc, addr, sizeof(struct pkt_primitives));
      elem_acc->packet_counter++;
      elem_acc->bytes_counter += pk_len;
      elem_acc->next = NULL;
      return (struct acc *) elem_acc;
    }
  }
}

