/*
    pmacct (Promiscuous mode IP Accounting package)
    pmacct is Copyright (C) 2003 by Paolo Lucente
*/

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#define __CFG_C

/* includes */
#include "pmacct.h"
#include "pmacct-data.h"
#include "plugin_hooks.h"

/* evaluate_configuration() handles all supported configuration
   keys and inserts them in the global configuration structure;
   much code has been taken from commandline option parsing in
   pmacctd.c */ 
int evaluate_configuration()
{
  char *ptr, *count_token;
  int index = 0;

  while (strcmp(_configuration[index].key, "")) {
    if (!strcmp(_configuration[index].key, "debug")) {
      if (!strcmp(_configuration[index].value, "true")) debug = TRUE;
    }
    else if (!strcmp(_configuration[index].key, "daemonize")) {
      if (!strcmp(_configuration[index].value, "true")) config.daemon = TRUE;
    }
    else if (!strcmp(_configuration[index].key, "aggregate")) { 
      if (strcmp(_configuration[index].value, "")) {
	ptr = _configuration[index].value;
	while (*ptr != '\0') {
          count_token = extract_token(&ptr, ',');
          if (!strcmp(count_token, "src_host")) config.what_to_count |= COUNT_SRC_HOST;
          else if (!strcmp(count_token, "dst_host")) config.what_to_count |= COUNT_DST_HOST;
          else if (!strcmp(count_token, "sum")) config.what_to_count |= COUNT_SUM_HOST;
          else if (!strcmp(count_token, "src_port")) config.what_to_count |= COUNT_SRC_PORT;
          else if (!strcmp(count_token, "dst_port")) config.what_to_count |= COUNT_DST_PORT;
          else if (!strcmp(count_token, "proto")) config.what_to_count |= COUNT_IP_PROTO;
          else if (!strcmp(count_token, "src_mac")) config.what_to_count |= COUNT_SRC_MAC;
          else if (!strcmp(count_token, "dst_mac")) config.what_to_count |= COUNT_DST_MAC;
          else printf("WARN: ignoring unknown aggregation method: %s.\n", count_token);
	}
      }
    }
    else if (!strcmp(_configuration[index].key, "pcap_filter")) {
      if (strcmp(_configuration[index].value, ""))
        config.clbuf = _configuration[index].value;
    }
    else if (!strcmp(_configuration[index].key, "plugins")) {
      if (strcmp(_configuration[index].value, ""))
	evaluate_plugins(_configuration[index].value);
    }
    else if (!strcmp(_configuration[index].key, "interface")) {
      if (strcmp(_configuration[index].value, ""))
        config.dev = _configuration[index].value;
    }
    else if (!strcmp(_configuration[index].key, "imt_path")) {
      if (strcmp(_configuration[index].value, ""))
	config.imt_plugin_path = _configuration[index].value;
    }
    else if (!strcmp(_configuration[index].key, "imt_buckets")) {
      if (strcmp(_configuration[index].value, ""))
	config.buckets = atoi(_configuration[index].value);
    }
    else if (!strcmp(_configuration[index].key, "imt_mem_pools_number")) {
      if (strcmp(_configuration[index].value, ""))
	config.num_memory_pools = atoi(_configuration[index].value);
    }
    else if (!strcmp(_configuration[index].key, "imt_mem_pools_size")) {
      if (strcmp(_configuration[index].value, ""))
       config.memory_pool_size = atoi(_configuration[index].value);
    }
    else if (!strcmp(_configuration[index].key, "sql_db")) {
      if (strcmp(_configuration[index].value, ""))
	config.sql_db = _configuration[index].value;
    }
    else if (!strcmp(_configuration[index].key, "sql_table")) {
      if (strcmp(_configuration[index].value, ""))
        config.sql_table = _configuration[index].value;
    }
    else if (!strcmp(_configuration[index].key, "sql_host")) {
      if (strcmp(_configuration[index].value, ""))
      config.sql_host = _configuration[index].value;
    }
    else if (!strcmp(_configuration[index].key, "sql_user")) {
      if (strcmp(_configuration[index].value, ""))
      config.sql_user = _configuration[index].value;
    }
    else if (!strcmp(_configuration[index].key, "sql_passwd")) {
      if (strcmp(_configuration[index].value, ""))
        config.sql_passwd = _configuration[index].value;
    }
    else if (!strcmp(_configuration[index].key, "sql_refresh_time")) {
      if (strcmp(_configuration[index].value, ""))
        config.sql_refresh_time = atoi(_configuration[index].value);
    }
    else if (!strcmp(_configuration[index].key, "sql_optimize_clauses")) {
      if (!strcmp(_configuration[index].value, "true"))
        config.sql_optimize_clauses = TRUE;
    }
    else if (!strcmp(_configuration[index].key, "sql_history")) {
      if (strcmp(_configuration[index].value, "")) {
	ptr = _configuration[index].value;
	parse_history_time(ptr, &config.sql_history, &config.sql_history_howmany); 
      }
    }
    else if (!strcmp(_configuration[index].key, "sql_history_roundoff")) {
      if (strcmp(_configuration[index].value, "")) 
	config.sql_history_roundoff = _configuration[index].value;
    }

    /* insert here new keys to handle */

    index++;
  }
  return SUCCESS;
}	

/* both parse_configuration_file() and parse_configuration_line()
   read configuration file and store its values (for supported keys)
   in a static in-memory structure for later processing */ 
int parse_configuration_file(char *filename)
{
  FILE *file;
  char line[SRVBUFLEN];
  int row = 0;

  /* if a config file is specified, command line options
     are discarded; but we restore the using_config_file
     switch */
  memset(&config, 0, sizeof(struct configuration));
  config.using_config_file = TRUE;

  if ((file = fopen(filename,"r")) == NULL) {
    printf("ERROR: file %s not found\n", filename);
    return ERR;
  }
  else {
    while (!feof(file)) {
      if (fgets(line, sizeof(line), file) == NULL) {
	fclose(file);
	if (debug) debug_configuration_file(); 
	return SUCCESS;
      }
      else {
        row++;
        if (parse_configuration_line(line) != SUCCESS)
	  printf("WARN: wrong syntax at line: %d\n", row);
      }
    }
  }
  fclose(file);
  if (debug) debug_configuration_file(); 
  return SUCCESS;
}

int parse_configuration_line(char *line)
{
  char *key, *value;
  int index = 0, len;

  /* few checks before parsing */
  if (line[0] == '!') return SUCCESS;  
  if (isblankline(line)) return SUCCESS;
  if (!strchr(line, ':')) return ERR;

  /* start parsing current line */
  key = (char *) strtok(line, ":");
  value = (char *) strtok(NULL, "\n");
  if ((!key) || (!value)) return ERR; 

  len = strlen(key); 
  if (len > (SRVBUFLEN-1)) return ERR;
  for (index = 0; index <= len; index++)
    key[index] = tolower(key[index]);

  len = strlen(value);
  if (len > (SRVBUFLEN-1)) return ERR;
  for (index = 0; index <= len; index++)
    value[index] = tolower(value[index]);
  
  trim_spaces(key);
  trim_spaces(value);
  
  for (index = 0; strcmp(_configuration[index].key, ""); index++) {
    if (!strcmp(key, _configuration[index].key)) {
      if (strcmp(_configuration[index].value, ""))
	printf("WARN: duplicate key (value of first occurrence is preserved): %s\n", key);
      else strlcpy(_configuration[index].value, value, sizeof(_configuration[index].value));
      return SUCCESS;
    }
  }
  printf("WARN: Ignoring unknown key: %s\n", key);
  return SUCCESS;
}

void debug_configuration_file()
{
  int index = 0;

  while (_configuration[index].key) {
    if (!strcmp(_configuration[index].key, "")) break; 
    printf("Key: %s Value: %s\n", _configuration[index].key, _configuration[index].value);
    index++;
  }
}

void parse_history_time(char *value, unsigned int *mu, unsigned int *howmany)
{
  int k, j, len;

  len = strlen(value);
  for (j = 0; j < len; j++) {
    if (!isdigit(value[j])) {
      if (value[j] == 'm') *mu = COUNT_MINUTELY; 
      else if (value[j] == 'h') *mu = COUNT_HOURLY; 
      else if (value[j] == 'd') *mu = COUNT_DAILY; 
      else {
	printf("WARN: ignoring unknown time measuring unit: '%c'.\n", value[j]);
	*mu = 0;
	*howmany = 0;
	return;
      }
      if (*mu) {
        value[j] = '\0';
	break;
      }
    }
  }
  k = atoi(value);
  if (k > 0) *howmany = k;
  else {
    printf("WARN: ignoring invalid time value: %d\n", k);
    *mu = 0;
    *howmany = 0;
  }
}
