/*
    pmacct (Promiscuous mode IP Accounting package)
    pmacct is Copyright (C) 2003 by Paolo Lucente
*/

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/* includes */
#include "pmacct.h"
#include "imt_plugin.h"

/* functions */
void init_memory_pool_table()
{
  if (config.num_memory_pools) {
    mpd = (unsigned char *) malloc((config.num_memory_pools+1)*sizeof(struct memory_pool_desc));
    memset(mpd, 0, (config.num_memory_pools+1)*sizeof(struct memory_pool_desc));
  }
  else {
    mpd = (unsigned char *) malloc((NUM_MEMORY_POOLS+1)*sizeof(struct memory_pool_desc));
    memset(mpd, 0, (NUM_MEMORY_POOLS+1)*sizeof(struct memory_pool_desc));
  }

  current_pool = NULL;
}


void clear_memory_pool_descriptors()
{
  unsigned char *mpd_ptr;
  struct memory_pool_desc *mp_desc, *mp_desc_next;

  mpd_ptr = mpd+sizeof(struct memory_pool_desc);
  mp_desc = (struct memory_pool_desc *) mpd_ptr;
  
  while (mp_desc->next != NULL) {
    free(mp_desc->base_ptr);
    if ((!config.num_memory_pools) && (mp_desc->id > NUM_MEMORY_POOLS)) {
      mp_desc_next = mp_desc->next;
      free(mp_desc);
      mp_desc = mp_desc_next;
    }
    else mp_desc = mp_desc->next; 
  }

  /* clearing last element of the list */
  free(mp_desc->base_ptr);
  if ((!config.num_memory_pools) && (mp_desc->id > NUM_MEMORY_POOLS)) 
    free(mp_desc);

  if (config.num_memory_pools) 
    memset(mpd, 0, (config.num_memory_pools+1)*sizeof(struct memory_pool_desc));
  else
    memset(mpd, 0, (NUM_MEMORY_POOLS+1)*sizeof(struct memory_pool_desc));

  current_pool = NULL;
}

struct memory_pool_desc *request_memory_pool(int size)
{
  int new_id; 
  unsigned char *desc, *mpd_ptr;
  struct memory_pool_desc *mp_desc;
  
  /* Allocating memory */
  desc = (unsigned char *) malloc(size);
  if (desc == NULL) {
    printf("WARN: We finished memory, clear in-memory stats !\n"); 
    return NULL;
  }

  memset(desc, 0, size);

  mpd_ptr = mpd;
  mp_desc = (struct memory_pool_desc *) mpd_ptr;
  new_id = 1;

  /* registering allocated memory in memory pool descriptors */
  while (mp_desc->next != NULL) {
    new_id++;
    mp_desc = mp_desc->next;
  }

  if (config.num_memory_pools) {
    if (new_id > config.num_memory_pools) return NULL; 
    mpd_ptr = mpd+(new_id*sizeof(struct memory_pool_desc));
    mp_desc->next = (struct memory_pool_desc *) mpd_ptr; 
    mp_desc = (struct memory_pool_desc *) mpd_ptr;
  }
  else {
    if (new_id > NUM_MEMORY_POOLS) {
      mpd_ptr = (unsigned char *) malloc(sizeof(struct memory_pool_desc));
      if (mpd_ptr == NULL) return NULL;
      memset(mpd_ptr, 0, sizeof(struct memory_pool_desc));
      mp_desc->next = (struct memory_pool_desc *) mpd_ptr;
      mp_desc = (struct memory_pool_desc *) mpd_ptr;
    }
    else {
      mpd_ptr = mpd+(new_id*sizeof(struct memory_pool_desc));
      mp_desc->next = (struct memory_pool_desc *) mpd_ptr;
      mp_desc = (struct memory_pool_desc *) mpd_ptr;
    }
  }

  mp_desc->id = new_id;
  mp_desc->base_ptr = desc;
  mp_desc->ptr = desc;
  mp_desc->space_left = size;
  mp_desc->len = size;
  return mp_desc;
}
