/*
    pmacct (Promiscuous mode IP Accounting package)
    pmacct is Copyright (C) 2003 by Paolo Lucente
*/

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* includes */
#include <mysql/mysql.h>
#include <sys/poll.h>

/* defines */
#define CACHE_ENTRIES 16385
#define QUERY_BUFFER_SIZE 32768 

/* macros */
#define SPACELEFT(x) (sizeof(x)-strlen(x)) 

/* structures */
struct db_cache {
  u_int8_t eth_dhost[ETH_ADDR_LEN];
  u_int8_t eth_shost[ETH_ADDR_LEN];
  struct in_addr ipa;
  struct in_addr flow;
  u_int16_t src_port;
  u_int16_t dst_port;
  u_int8_t proto;
  int packet_counter;
  int bytes_counter;
  time_t basetime;
  int valid;
};

typedef void (*dbop_handler) (const struct db_cache *, int, char **, char **);

struct frags {
  dbop_handler handler;
  unsigned long int type;
  char string[SRVBUFLEN];
};

struct insert_data {
  unsigned long int modulo;
  time_t basetime;
  time_t timeslot;
  int num_primitives;
};

/* prototypes */
#if (!defined __MYSQL_PLUGIN_C)
#define EXT extern
#else
#define EXT
#endif

EXT void mysql_plugin(int);
EXT void db_error_handling(MYSQL *);
EXT unsigned int cache_modulo(struct pkt_primitives *);
EXT void cache_insert(struct pkt_primitives *, unsigned int, struct insert_data *);
EXT void cache_aliasing(const struct db_cache *, const int);
EXT void cache_dbop(MYSQL *, const struct db_cache *, const int);
EXT void cache_purge(struct db_cache *[], int, const int);
EXT int cache_flush(struct db_cache *[], int);
EXT int evaluate_history(int);
EXT int evaluate_primitives(int);
EXT int compose_static_queries();
EXT void exit_gracefully(int);
#undef EXT

#if (!defined __MYSQL_PLUGIN_HANDLERS_C)
#define EXT extern
#else
#define EXT
#endif
EXT void count_src_mac_handler(const struct db_cache *, int, char **, char **); 
EXT void count_dst_mac_handler(const struct db_cache *, int, char **, char **); 
EXT void count_src_host_handler(const struct db_cache *, int, char **, char **); 
EXT void count_dst_host_handler(const struct db_cache *, int, char **, char **); 
EXT void count_src_port_handler(const struct db_cache *, int, char **, char **); 
EXT void count_dst_port_handler(const struct db_cache *, int, char **, char **); 
EXT void count_ip_proto_handler(const struct db_cache *, int, char **, char **); 
EXT void count_timestamp_handler(const struct db_cache *, int, char **, char **); 
#undef EXT

/* global vars: many of them are meant to speed-up the wheel */
#if (!defined __MYSQL_PLUGIN_C)
#define EXT extern
#else
#define EXT
#endif 

EXT MYSQL db_desc;
EXT char mysql_data[LONGLONGSRVBUFLEN];
EXT char lock_clause[LONGSRVBUFLEN];
EXT char unlock_clause[LONGSRVBUFLEN];
EXT char update_clause[LONGSRVBUFLEN];
EXT char insert_clause[LONGSRVBUFLEN];
EXT char values_clause[LONGSRVBUFLEN];
EXT char where_clause[LONGSRVBUFLEN];
EXT struct db_cache cache[CACHE_ENTRIES];
EXT struct db_cache *queries_queue[CACHE_ENTRIES];
EXT int qq_ptr, pp_size;
EXT struct frags where[N_PRIMITIVES+2];
EXT struct frags values[N_PRIMITIVES+2];
EXT int num_primitives; /* last resort for signal handling */
#undef EXT
