/*
    pmacct (Promiscuous mode IP Accounting package)
    pmacct is Copyright (C) 2003 by Paolo Lucente
*/

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#define __MYSQL_PLUGIN_HANDLERS_C

/* includes */
#include "pmacct.h"
#include "pmacct-data.h"
#include "plugin_hooks.h"
#include "mysql_plugin.h"
#include "util.h"

/* Functions */
void count_src_mac_handler(const struct db_cache *cache_elem, int num, char **ptr_values, char **ptr_where)
{
  char *ptr;

  ptr = (char *)ether_ntoa(cache_elem->eth_shost);
  snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, ptr);
  snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, ptr);
  *ptr_where = where_clause + strlen(where_clause);
  *ptr_values = values_clause + strlen(values_clause);
}

void count_dst_mac_handler(const struct db_cache *cache_elem, int num, char **ptr_values, char **ptr_where)
{
  char *ptr;

  ptr = (char *)ether_ntoa(cache_elem->eth_dhost);
  snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, ptr);
  snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, ptr);
  *ptr_where = where_clause + strlen(where_clause);
  *ptr_values = values_clause + strlen(values_clause);
}

void count_src_host_handler(const struct db_cache *cache_elem, int num, char **ptr_values, char **ptr_where)
{
  char *ptr;

  ptr = inet_ntoa(cache_elem->ipa);
  snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, ptr);
  snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, ptr);
  *ptr_where = where_clause + strlen(where_clause);
  *ptr_values = values_clause + strlen(values_clause);
}

void count_dst_host_handler(const struct db_cache *cache_elem, int num, char **ptr_values, char **ptr_where)
{
  char *ptr;

   ptr = inet_ntoa(cache_elem->flow);
   snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, ptr);
   snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, ptr);
   *ptr_where = where_clause + strlen(where_clause);
   *ptr_values = values_clause + strlen(values_clause);
}

void count_src_port_handler(const struct db_cache *cache_elem, int num, char **ptr_values, char **ptr_where)
{
  char *ptr;

  snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, cache_elem->src_port);
  snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, cache_elem->src_port);
  *ptr_where = where_clause + strlen(where_clause);
  *ptr_values = values_clause + strlen(values_clause);
}

void count_dst_port_handler(const struct db_cache *cache_elem, int num, char **ptr_values, char **ptr_where)
{
  char *ptr;
      
  snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, cache_elem->dst_port);
  snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, cache_elem->dst_port);
  *ptr_where = where_clause + strlen(where_clause);
  *ptr_values = values_clause + strlen(values_clause);
}

void count_ip_proto_handler(const struct db_cache *cache_elem, int num, char **ptr_values, char **ptr_where)
{
  char *ptr;

  if (cache_elem->proto <= protocols_number) {
    snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, _protocols[cache_elem->proto].name);
    snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, _protocols[cache_elem->proto].name);
  }
  else {
    snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, cache_elem->proto);
    snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, cache_elem->proto);
  }
  *ptr_where = where_clause + strlen(where_clause);
  *ptr_values = values_clause + strlen(values_clause);
}

void count_timestamp_handler(const struct db_cache *cache_elem, int num, char **ptr_values, char **ptr_where)
{
  char *ptr;

  snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, cache_elem->basetime);
  snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, cache_elem->basetime);
  *ptr_where = where_clause + strlen(where_clause);
  *ptr_values = values_clause + strlen(values_clause);
}

