/*
    pmacct (Promiscuous mode IP Accounting package)
    pmacct is Copyright (C) 2003 by Paolo Lucente
*/

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if no, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#define __PLUGIN_HOOKS_C

/* includes */
#include "pmacct.h"
#include "plugin_hooks.h"

/* functions */
void evaluate_plugins(char *plugins)
{
  int delim = ',';
  char *token;

  /* manually specifying plugins to enable;
     we have to disable default setting */
  use_inmemory_table = FALSE;

  while (*plugins != '\0') { 
    token = extract_token(&plugins, delim); 
    if (token != NULL) {
      if (strcmp(token, "memory") == 0)
	use_inmemory_table = TRUE;
#ifdef WITH_MYSQL
      else if (strcmp(token, "mysql") == 0)
	use_mysql_db = TRUE; 
#endif
      else {
	printf("ERROR: %s plugin is not loaded or unknown\n", token); 
	exit(1);
      }
    }
  }
}

char *extract_token(char **string, int delim)
{
  char *token, *delim_ptr;

  if ((delim_ptr = strchr(*string, delim))) {
    *delim_ptr = '\0';
    token = *string;
    *string = delim_ptr+1;
  }
  else {
    token = *string;
    *string += strlen(*string);
  }

  return token;
}

void register_plugin(int child_pid)
{
  int index = 0;

  while (index < MAX_N_PLUGINS) {
    if (!config.registered_plugins[index]) {
      config.registered_plugins[index] = child_pid;
      break;
    }
    else index++;
  }
}

void load_plugins()
{
  int pid;

  if (use_inmemory_table) {
    config.active_plugins++;
    pipe(imt_plugin_pipe);
    switch (pid = fork()) {
    case 0: /* Child */
      close(imt_plugin_pipe[1]);
      setnonblocking(imt_plugin_pipe[0]);
      imt_plugin(imt_plugin_pipe[0]); 
      exit(0);
    default: /* Parent */
      register_plugin(pid);
      close(imt_plugin_pipe[0]);
      setnonblocking(imt_plugin_pipe[1]);
    }
  }
#ifdef WITH_MYSQL
  if (use_mysql_db) {
    config.active_plugins++;
    pipe(mysql_plugin_pipe);
    switch (pid = fork()) {
    case 0: /* Child */
      close(mysql_plugin_pipe[1]);
      mysql_plugin(mysql_plugin_pipe[0]);
      exit(0);
    default: /* Parent */
      register_plugin(pid);
      close(mysql_plugin_pipe[0]);
      setnonblocking(mysql_plugin_pipe[1]);
    }
  }
#endif 
}

void exec_plugins(struct packet_ptrs *pptrs) 
{
  char buf[SRVBUFLEN];
  struct pkt_data *pdata;
  struct my_iphdr *iph_ptr; 
  struct my_tlhdr *tlh_ptr;

  /* constructing buffer: supported primitives + packet total length */
  pdata = (struct pkt_data *) buf;
  iph_ptr = (struct my_iphdr *) pptrs->iph_ptr;
  tlh_ptr = (struct my_tlhdr *) pptrs->tlh_ptr;
  memcpy(pdata, pptrs->mac_ptr, (2*ETH_ADDR_LEN)); 
  pdata->primitives.ipa.s_addr = iph_ptr->ip_src.s_addr;
  pdata->primitives.flow.s_addr = iph_ptr->ip_dst.s_addr;
  pdata->primitives.proto = iph_ptr->ip_p;
  pdata->primitives.src_port = tlh_ptr->src_port;
  pdata->primitives.dst_port = tlh_ptr->dst_port;
  pdata->pkt_len = iph_ptr->ip_len; 
  
  /* writes data to in-memory table plugin */
  if (use_inmemory_table) {
    if (imt_plugin_pipe[1] != ERR) {
      if (write(imt_plugin_pipe[1], buf, sizeof(struct pkt_data)) == -1) {
        printf("ERROR: lost connection to IMT plugin; closing connection (%d) ...\n", errno);
        close(imt_plugin_pipe[1]);
        imt_plugin_pipe[1] = ERR;
        config.active_plugins--;
        if (!config.active_plugins) {
 	  printf("ERROR: no more plugins active. Shutting down ...\n");
	  exit(1);
        }
      }
    }
  }

#ifdef WITH_MYSQL
  /* writes data to mysql database plugin */
  if (use_mysql_db) {
    if (mysql_plugin_pipe[1] != ERR) {
      if (write(mysql_plugin_pipe[1], buf, sizeof(struct pkt_data)) == -1) {
	if (errno == EAGAIN);
	else {
          printf("ERROR: lost connection to MySQL plugin; closing connection (%d) ...\n", errno);
          close(mysql_plugin_pipe[1]);
          mysql_plugin_pipe[1] = ERR;
          config.active_plugins--;
	  if (!config.active_plugins) {
	    printf("ERROR: no more plugins active. Shutting down ...\n");
	    exit(1);
	  }
	}	
      }
    }
  }
#endif
}
